@@ -4,6 +4,8 @@ import decimal
 import datetime
 import uuid
 import pyrfc3339
+from edn_format.immutable_dict import ImmutableDict
+
 from .edn_lex import Keyword, Symbol
 from .edn_parse import TaggedElement
 
@@ -37,9 +39,9 @@ def dump(obj):
         return "({})".format(seq(obj))
     elif isinstance(obj, list):
         return "[{}]".format(seq(obj))
-    elif isinstance(obj, set):
+    elif isinstance(obj, set) or isinstance(obj, frozenset):
         return "#{{{}}}".format(seq(obj))
-    elif isinstance(obj, dict):
+    elif isinstance(obj, dict) or isinstance(obj, ImmutableDict):
         return "{{{}}}".format(seq(itertools.chain.from_iterable(obj.items())))
     elif isinstance(obj, datetime.datetime):
         return '#inst "{}"'.format(pyrfc3339.generate(obj))
