@@ -4,6 +4,7 @@ import ply.lex
 import logging
 import re
 import decimal
+from immutable_dict import ImmutableDict
 
 
 if sys.version_info[0] == 3:
@@ -19,7 +20,10 @@ class BaseEdnType(object):
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
             return False
-        return repr(self) == repr(other)
+        return self.__dict__ == other.__dict__
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
 
     def __ne__(self, other):
         return not self.__eq__(other)
@@ -28,15 +32,23 @@ class BaseEdnType(object):
         return "{}({})".format(self.__class__.__name__, self._name)
 
     def __hash__(self):
-        return hash(repr(self))
+        return ImmutableDict(self.__dict__).__hash__()
 
 
 class Keyword(BaseEdnType):
+    def __init__(self, name):
+        self._name = unicode(name)
+        self._type = Keyword
+
     def __str__(self):
         return ":{}".format(self._name)
 
 
 class Symbol(BaseEdnType):
+    def __init__(self, name):
+        self._name = unicode(name)
+        self._type = Symbol
+
     def __str__(self):
         return self._name
 
@@ -131,9 +143,8 @@ def t_CHAR(t):
     t.value = t.value[1]
     return t
 
-
 def t_STRING(t):
-    r'"(.|\"|\t|\n|\r|\newline|\return|\space|\tab)*"'
+    r"\"(\\.|[^\"])*\""
     t.value = t.value[1:-1]
     t.value = t.value.replace(r"\newline", "\n") \
                      .replace(r"\return", "\r") \
@@ -209,7 +220,7 @@ def t_SYMBOL(t):
 
 
 def t_error(t):
-    raise SyntaxError("Illegal character '%s'" % t.value[0])
+    raise SyntaxError("Illegal character '%s' with lexpos %s in the area of -  %s  -" % (t.value[0], t.lexpos, t.value[0:100]))
 
 
 def lex(text=None):
