@@ -6,6 +6,7 @@ import pyrfc3339
 
 import ply.yacc
 from .edn_lex import tokens, lex
+from immutable_dict import ImmutableDict
 
 if sys.version_info[0] == 3:
     long = int
@@ -18,7 +19,7 @@ if tokens:
 
 start = 'expression'
 
-_serializers = {}
+_serializers = dict({})
 
 
 class TaggedElement(object):
@@ -74,17 +75,17 @@ def p_list(p):
 
 def p_empty_set(p):
     """set : SET_START MAP_OR_SET_END"""
-    p[0] = set()
+    p[0] = frozenset()
 
 
 def p_set(p):
     """set : SET_START expressions MAP_OR_SET_END"""
-    p[0] = set(p[2])
+    p[0] = frozenset(p[2])
 
 
 def p_empty_map(p):
     """map : MAP_START MAP_OR_SET_END"""
-    p[0] = {}
+    p[0] = ImmutableDict({})
 
 
 def p_map(p):
@@ -93,7 +94,7 @@ def p_map(p):
     if len(terms) % 2 != 0:
         raise SyntaxError("Even number of terms required for map")
     # partition terms in pairs
-    p[0] = dict([terms[i:i+2] for i in range(0, len(terms), 2)])
+    p[0] = ImmutableDict(dict([terms[i:i+2] for i in range(0, len(terms), 2)]))
 
 
 def p_expressions_expressions_expression(p):
@@ -127,8 +128,7 @@ def p_expression_tagged_element(p):
     elif tag in _serializers:
         output = _serializers[tag](element)
     else:
-        raise NotImplementedError(
-            "Don't know how to handle tag {}".format(tag))
+        raise NotImplementedError("Don't know how to handle tag ImmutableDict({})".format(tag))
 
     p[0] = output
 
@@ -141,9 +141,8 @@ def p_error(p):
 
 
 def parse(text):
-    kwargs = {}
+    kwargs = ImmutableDict({})
     if __debug__:
-        kwargs = dict(debug=True)
+        kwargs = dict({"debug": True})
     p = ply.yacc.yacc(**kwargs)
-
     return p.parse(text, lexer=lex())
