@@ -1,14 +1,17 @@
 import datetime
 import pytz
 import unittest
-from edn_format import edn_lex, \
-    edn_parse, \
-    loads, \
-    dumps, \
-    Symbol, \
-    Keyword, \
-    TaggedElement, \
-    add_tag
+from edn_format import edn_lex, edn_parse, \
+     loads, dumps, Keyword, Symbol, TaggedElement, add_tag
+
+class ConsoleTest(unittest.TestCase):
+    def test_dumping(self):
+        is_exception = False
+        try:
+            loads("[1 true nil]")
+        except AttributeError as x:
+            is_exception = True
+        self.assertFalse(is_exception)
 
 
 class EdnTest(unittest.TestCase):
@@ -63,8 +66,8 @@ class EdnTest(unittest.TestCase):
                          'a*b')
         self.check_parse("ab",
                          '"ab"')
-        self.check_parse("a\"b",
-                         '"a\"b"')
+        self.check_parse("a\\\"b",
+                         '"a\\"b"')
         self.check_parse("blah\n",
                          '"blah\n"')
         self.check_parse("blah blah",
@@ -91,22 +94,21 @@ class EdnTest(unittest.TestCase):
         self.check_parse([], "[]")
         self.check_parse({"a": [1, 2, 3]},
                          '{"a" [1 2 3]}')
-        self.check_parse(
-            datetime.datetime(2012, 12, 22, 19, 40, 18, 0, tzinfo=pytz.utc),
-            '#inst "2012-12-22T19:40:18Z"')
+        self.check_parse(datetime.datetime(2012, 12, 22, 19, 40, 18, 0, tzinfo=pytz.utc),
+                        '#inst "2012-12-22T19:40:18Z"')
+        self.check_parse("|", "\"|\"")
+        self.check_parse("%", "\"%\"")
+        self.check_parse(['bl\\"ah'], """["bl\\"ah"]""")
+        self.check_parse("blah\n", '"blah\n"')
+        self.check_parse(["abc", "123"], '["abc", "123"]')
+        self.check_parse({"key": "value"}, '{"key" "value"}')
 
-    def check_dump(self, expected_output, actual_input):
-        if isinstance(expected_output, list):
-            self.assertIn(dumps(actual_input), expected_output)
-        else:
-            self.assertEqual(expected_output, dumps(actual_input))
+    def check_roundtrip(self, data_input):
+        self.assertEqual(data_input, loads(dumps(data_input)))
 
     def test_dump(self):
-        self.check_dump("#{1 2 3}",
-                        {1, 2, 3})
-        self.check_dump(
-            ['{:bar [1 2 3] :a 1 "foo" :gone}',
-             '{:a 1 "foo" :gone :bar [1 2 3]}'],
+        self.check_roundtrip({1, 2, 3})
+        self.check_roundtrip(
             {Keyword("a"): 1,
              "foo": Keyword("gone"),
              Keyword("bar"): [1, 2, 3]})
@@ -174,9 +176,9 @@ class EdnTest(unittest.TestCase):
             '#inst "1985-04-12T23:20:50Z"',
             '#uuid "f81d4fae-7dec-11d0-a765-00a0c91e6bf6"',
             '#date "19/07/1984"',
-            ## http://git.io/wQy2_g
-            ## (part of https://github.com/shaunxcode/edn-tests)
-            # '#myapp/Person {:first "Fred" :last "Mertz"}',
+            '#{{"a" 1}}',
+            '#{{"a" #{{:b 2}}}}',
+            '"|"'
         )
 
         class TagDate(TaggedElement):
@@ -200,6 +202,11 @@ class EdnTest(unittest.TestCase):
 #            print step1, "->", step2, "->", step3
             self.assertEqual(step1, step3)
 
+class EdnInstanceTest(unittest.TestCase):
+    def test_hashing(self):
+        pop_count = len(set(map(hash, ["db/id", Keyword("db/id"), Symbol("db/id")])))
+        self.assertEqual(pop_count, 3)
+
     def test_equality(self):
         self.assertTrue("db/id" != Keyword("db/id"))
         self.assertTrue("db/id" != Symbol("db/id"))
